@@ -7,9 +7,11 @@ import (
 	"math/big"
 	"mime"
 	"net"
+	"os"
 	"strings"
 
-	"github.com/howeyc/gopass"
+	"github.com/grandcat/zeroconf"
+	"github.com/patrickhener/goshs/goshsversion"
 	"github.com/patrickhener/goshs/logger"
 	"golang.org/x/crypto/bcrypt"
 )
@@ -96,26 +98,92 @@ func GetAllIPAdresses() (map[string]string, error) {
 
 // GenerateHashPassword will take a plaintext masked password and return a bcrypt hash
 // This is meant to be used with the filebased access via .goshs file
-func GenerateHashedPassword() {
-	fmt.Printf("Enter password: ")
-	password, err := gopass.GetPasswdMasked()
-	if err != nil {
-		logger.Fatalf("error reading password from stdin: %+v", err)
-	}
-
-	bytes, err := bcrypt.GenerateFromPassword([]byte(password), 14)
+func GenerateHashedPassword(password []byte) string {
+	bytes, err := bcrypt.GenerateFromPassword(password, 14)
 	if err != nil {
 		logger.Fatalf("error hashing password: %+v", err)
 	}
 	fmt.Printf("Hash: %s\n", string(bytes))
+
+	return string(bytes)
 }
 
-// Contains checks if a string is in a slice of strings
-func Contains(slice []string, item string) bool {
-	for _, a := range slice {
-		if a == item {
-			return true
-		}
+func RegisterZeroconfMDNS(ssl bool, webPort int, webdav bool, webdavPort int, sftp bool, sftpPort int) error {
+	// Register zeroconf mDNS
+	hostname, err := os.Hostname()
+	if err != nil {
+		return fmt.Errorf("cannot get hostname for mDNS: %+v", err)
 	}
-	return false
+	// Register webPort
+	var serviceType string
+	var out string
+	switch ssl {
+	case true:
+		serviceType = "_https._tcp"
+		out = "https"
+	default:
+		serviceType = "_http._tcp"
+		out = "http"
+	}
+	zero, err := zeroconf.Register(
+		"goshs WebInterface",
+		serviceType,
+		"local.",
+		webPort,
+		[]string{fmt.Sprintf("host=%s.local", hostname), "path=/", fmt.Sprintf("version=%s", goshsversion.GoshsVersion)},
+		nil,
+	)
+	if err != nil {
+		return fmt.Errorf("zeroconf mDNS did not register successfully: %+v", err)
+	}
+	defer zero.Shutdown()
+
+	logger.Infof("mDSN service registered as %s://%s.local:%d", out, hostname, webPort)
+
+	// Register webdav if enabled
+	if webdav {
+		switch ssl {
+		case true:
+			serviceType = "_webdavs._tcp"
+			out = "webdavs"
+		default:
+			serviceType = "_webdav._tcp"
+			out = "webdav"
+		}
+
+		zeroDav, err := zeroconf.Register(
+			"goshs WebDAV",
+			serviceType,
+			"local.",
+			webdavPort,
+			[]string{fmt.Sprintf("host=%s.local", hostname), "path=/", fmt.Sprintf("version=%s", goshsversion.GoshsVersion)},
+			nil,
+		)
+		if err != nil {
+			return fmt.Errorf("zeroconf mDNS did not register successfully: %+v", err)
+		}
+		defer zeroDav.Shutdown()
+
+		logger.Infof("mDSN service registered as %s://%s.local:%d", out, hostname, webdavPort)
+	}
+
+	// Register sftp if enabled
+	if sftp {
+		zeroDav, err := zeroconf.Register(
+			"goshs WebDAV",
+			"_ssh._tcp",
+			"local.",
+			webdavPort,
+			[]string{fmt.Sprintf("host=%s.local", hostname), "subsystem=sftp", "path=/", fmt.Sprintf("version=%s", goshsversion.GoshsVersion)},
+			nil,
+		)
+		if err != nil {
+			return fmt.Errorf("zeroconf mDNS did not register successfully: %+v", err)
+		}
+		defer zeroDav.Shutdown()
+
+		logger.Infof("mDSN service registered as ssh://%s.local:%d", hostname, sftpPort)
+	}
+
+	return nil
 }
